@@ -387,7 +387,7 @@ void** IntMapGetRef(IntMap_p map, long key)
             val = map->values.value;
             map->values.tree = NULL;
             tmp.p_val = val;
-            NumTreeStore(&(map->values.tree),map->max_key, tmp, tmp);
+            NumTreeStore(&(map->values.tree), map->max_key, tmp, tmp);
             handle = add_new_tree_node(map, key, NULL);
             res = &(handle->val1.p_val);
             map->entry_no = 2;
@@ -653,7 +653,7 @@ void IntMapIterFree(IntMapIter_p junk)
 
 void IntMapDebugPrint(FILE* out, IntMap_p map)
 {
-   IntMapIter_p iter = IntMapIterAlloc(map,0, LONG_MAX);
+   IntMapIter_p iter = IntMapIterAlloc(map, LONG_MIN, LONG_MAX);
    void* val;
    long  key = 0;
 
@@ -673,5 +673,3 @@ void IntMapDebugPrint(FILE* out, IntMap_p map)
 /*---------------------------------------------------------------------*/
 /*                        End of File                                  */
 /*---------------------------------------------------------------------*/
-
-
