@@ -326,12 +326,13 @@ static bool eqn_parse_prefix(Scanner_p in, TB_p bank, Term_p *lref,
    }
    if(rterm == bank->true_term)
    {
-      if(TermIsVar(lterm))
+      if(TermIsFreeVar(lterm))
       {
          AktTokenError(in, "Individual variable "
                      "used at predicate position", false);
 
       }
+      assert(!(TermIsDBVar(lterm)));
       SigDeclareIsPredicate(bank->sig, lterm->f_code);
    }
    *lref = lterm;
@@ -474,9 +475,9 @@ bool EqnParseInfix(Scanner_p in, TB_p bank, Term_p *lref, Term_p *rref)
    BOOL_TERM_NORMALIZE(lterm);
 
    /* Shortcut not to check for equality --
-         !TermIsVar guards calls against negative f_code */
-   if(problemType == PROBLEM_FO && !TermIsVar(lterm) &&
-      SigIsPredicate(bank->sig,lterm->f_code) &&
+         !TermIsFreeVar guards calls against negative f_code */
+   if(problemType == PROBLEM_FO && !TermIsFreeVar(lterm) &&
+      SigIsPredicate(bank->sig, lterm->f_code) &&
       SigIsFixedType(bank->sig, lterm->f_code))
    {
       rterm = bank->true_term; /* Non-Equational literal */
@@ -484,9 +485,9 @@ bool EqnParseInfix(Scanner_p in, TB_p bank, Term_p *lref, Term_p *rref)
    else
    {
       /* If we have a predicate variable then = might not come */
-      if((TermIsVar(lterm) && !TypeIsPredicate(lterm->type))
+      if((TermIsAnyVar(lterm) && !TypeIsPredicate(lterm->type))
               /* guarding SigIsFunction */
-          || (!TermIsVar(lterm) && SigIsFunction(bank->sig, lterm->f_code)))
+          || (!TermIsAnyVar(lterm) && SigIsFunction(bank->sig, lterm->f_code)))
       {
          if(in_parens && TestInpTok(in, CloseBracket))
          {
@@ -496,9 +497,18 @@ bool EqnParseInfix(Scanner_p in, TB_p bank, Term_p *lref, Term_p *rref)
 
          if(!TestInpTok(in, NegEqualSign|EqualSign))
          {
+#ifdef ENABLE_LFHO
             // type is known but it is inside $let
             // or $ite and is not in an equation
-            rterm = NULL;
+            // (StS):I would think this is an actual parse
+            // error...Petar set this to NULL, which crashes on some
+            // garbage input in LOP format.
+            rterm = bank->true_term;
+#else
+            CheckInpTok(in, NegEqualSign|EqualSign);
+            rterm = NULL; //Stiffle warning
+#endif
+
          }
          else
          {
@@ -510,7 +520,7 @@ bool EqnParseInfix(Scanner_p in, TB_p bank, Term_p *lref, Term_p *rref)
 
             rterm = TBTermParse(in, bank);
 
-            if(!TermIsTopLevelVar(rterm))
+            if(!TermIsTopLevelAnyVar(rterm))
             {
                TypeDeclareIsNotPredicate(bank->sig, rterm, in);
             }
@@ -548,7 +558,9 @@ bool EqnParseInfix(Scanner_p in, TB_p bank, Term_p *lref, Term_p *rref)
       }
       else
       {  /* It's a predicate */
-         if(problemType == PROBLEM_HO && !TermIsTopLevelVar(lterm)
+         if(problemType == PROBLEM_HO
+            && !TermIsAnyVar(lterm)
+            && !TermIsPhonyApp(lterm)
             && SigIsFunction(bank->sig, lterm->f_code))
          {
             DStr_p err = DStrAlloc();
@@ -557,10 +569,20 @@ bool EqnParseInfix(Scanner_p in, TB_p bank, Term_p *lref, Term_p *rref)
             DStrAppendStr(err, " interpreted both as function and predicate (check parentheses).");
             AktTokenError(in, DStrView(err), SYNTAX_ERROR);
          }
-         rterm = bank->true_term; /* Non-Equational literal */
-         if(lterm->f_code > bank->sig->internal_symbols)
+         if(TypeIsBool(lterm->type) ||
+            (!TermIsFreeVar(lterm) &&
+             lterm->f_code > bank->sig->internal_symbols &&
+             !SigIsFixedType(bank->sig, lterm->f_code))) // can be special symbol like $ite and $let
          {
-            TypeDeclareIsPredicate(bank->sig, lterm);
+            rterm = bank->true_term; /* Non-Equational literal */
+            if(lterm->f_code > bank->sig->internal_symbols)
+            {
+               TypeDeclareIsPredicate(bank->sig, lterm);
+            }
+         }
+         else
+         {
+            rterm = NULL;
          }
       }
    }
@@ -591,6 +613,20 @@ bool EqnParseInfix(Scanner_p in, TB_p bank, Term_p *lref, Term_p *rref)
 Eqn_p EqnAlloc(Term_p lterm, Term_p rterm, TB_p bank,  bool positive)
 {
    Eqn_p handle = EqnCellAlloc();
+   if(lterm == bank->false_term)
+   {
+      lterm = bank->true_term;
+      positive = !positive;
+   }
+   if(rterm == bank->false_term)
+   {
+      rterm = bank->true_term;
+      positive = !positive;
+   }
+   if(lterm == bank->true_term)
+   {
+      SWAP(lterm, rterm);
+   }
 
    /* printf("Handle = %p\n", handle); */
    handle->properties = EPNoProps;
@@ -611,15 +647,16 @@ Eqn_p EqnAlloc(Term_p lterm, Term_p rterm, TB_p bank,  bool positive)
                SigPrint(stdout,bank->sig);
                fflush(stdout); */
 #ifndef ENABLE_LFHO
-      //assert(!TermIsVar(lterm));
+      //assert(!TermIsFreeVar(lterm));
 #endif
       if(lterm->f_code > bank->sig->internal_symbols)
       {
+         assert(!TermIsDBVar(lterm));
          SigDeclareIsPredicate(bank->sig, lterm->f_code);
       }
 
       TermCellSetProp(lterm, TPPredPos);
-      if(!TermIsVar(lterm) && SigQueryFuncProp(bank->sig, lterm->f_code, FPPseudoPred))
+      if(!TermIsAnyVar(lterm) && SigQueryFuncProp(bank->sig, lterm->f_code, FPPseudoPred))
       {
          EqnSetProp(handle, EPPseudoLit);
       }
@@ -627,23 +664,59 @@ Eqn_p EqnAlloc(Term_p lterm, Term_p rterm, TB_p bank,  bool positive)
 
    /* Allowing predicate variables to be stored in equation */
    if(lterm->type != rterm->type &&
-         !(TypeIsPredicate(lterm->type) && rterm == bank->true_term))
+      !(TypeIsPredicate(lterm->type) && rterm == bank->true_term))
    {
       TermAssertSameSort(bank->sig, lterm, rterm);
    }
-
    handle->bank = bank;
    handle->next = NULL;
 
    handle->lterm = lterm;
    handle->rterm = rterm;
 
+
    /* EqnPrint(stdout, handle, false, true);
       printf("\n"); */
 
    return handle;
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: EqnAllocFlatten()
+//
+//   Allocates a predicate literal but makes sure that if it is
+//   a formula of the form (~)$eq(s,t) then s and t are lifted
+//   to the literal level.
+//
+// Global Variables: -
+//
+// Side Effects    : Adds references
+//
+/----------------------------------------------------------------------*/
+
+Eqn_p EqnAllocFlatten(Term_p lterm, TB_p bank, bool sign)
+{
+   assert(TypeIsBool(lterm->type));
+   Eqn_p res;
+   if(lterm->f_code == bank->sig->eqn_code ||
+      lterm->f_code == bank->sig->neqn_code)
+   {
+      if(lterm->f_code == bank->sig->neqn_code)
+      {
+         sign = !sign;
+      }
+      res = EqnAlloc(lterm->args[0], lterm->args[1], bank, sign);
+   }
+   else
+   {
+      res = EqnAlloc(lterm, bank->true_term, bank, sign);
+   }
+   return res;
+}
+
+
+
 
 /*-----------------------------------------------------------------------
 //
@@ -716,7 +789,7 @@ Eqn_p EqnFOFParse(Scanner_p in, TB_p bank)
    Term_p lterm, rterm;
    Eqn_p handle;
 
-   
+
    positive = eqn_parse_real(in, bank, &lterm, &rterm, true);
    handle = EqnAlloc(lterm, rterm, bank, positive);
 
@@ -787,7 +860,8 @@ Eqn_p EqnHOFParse(Scanner_p in, TB_p bank, bool* continue_parsing)
    }
    else
    {
-      if(!TermIsTopLevelVar(lterm) && SigIsFunction(bank->sig, lterm->f_code))
+      if(!TermIsAnyVar(lterm) && !TermIsPhonyApp(lterm)
+         && SigIsFunction(bank->sig, lterm->f_code))
       {
          DStr_p err = DStrAlloc();
          DStrAppendStr(err, "Symbol ");
@@ -968,6 +1042,7 @@ void EqnPrint(FILE* out, Eqn_p eq, bool negated,  bool fullterms)
           /* || eq->lterm==eq->bank->true_term*/
             ))
       {
+         PRINT_HO_PAREN(out, '(');
          TBPrintTerm(out, eq->bank, eq->lterm, fullterms);
 
          if(!positive)
@@ -977,6 +1052,7 @@ void EqnPrint(FILE* out, Eqn_p eq, bool negated,  bool fullterms)
          /* fprintf(out, EqnIsOriented(eq)?"=>":"="); */
          fprintf(out, "=");
          TBPrintTerm(out, eq->bank, eq->rterm, fullterms);
+         PRINT_HO_PAREN(out, ')');
       }
       else
       {
@@ -996,7 +1072,9 @@ void EqnPrint(FILE* out, Eqn_p eq, bool negated,  bool fullterms)
          }
          else
          {
+            PRINT_HO_PAREN(out, '(');
             TBPrintTerm(out, eq->bank, eq->lterm, fullterms);
+            PRINT_HO_PAREN(out, ')');
          }
       }
    }
@@ -1008,6 +1086,25 @@ void EqnPrint(FILE* out, Eqn_p eq, bool negated,  bool fullterms)
 #endif
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: EqnPrintDBG()
+//
+//   Debug printing of the equation.
+//
+// Side Effects    : Output
+//
+/----------------------------------------------------------------------*/
+void EqnPrintDBG(FILE* out, Eqn_p eq)
+{
+   TermPrintDbg(out, eq->lterm, eq->bank->sig, DEREF_NEVER);
+   fprintf(out, "%s=", EqnIsPositive(eq)?"":"!");
+   TermPrintDbg(out, eq->rterm, eq->bank->sig, DEREF_NEVER);
+   fprintf(out, "%s", EqnIsMaximal(eq) ? "*" : "");
+   fprintf(out, "%s", EqnIsOriented(eq) ? ">" : "");
+   fprintf(out, "%s", EqnQueryProp(eq, EPIsEquLiteral) ? "#" : "");
+}
+
 /*-----------------------------------------------------------------------
 //
 // Function: EqnPrintDeref()
@@ -1065,6 +1162,7 @@ void EqnFOFPrint(FILE* out, Eqn_p eq, bool negated,  bool fullterms, bool pcl)
    {
       if(EqnIsEquLit(eq))
       {
+         PRINT_HO_PAREN(out, '(');
          PRINT_HO_PAREN(out, '(');
          TBPrintTerm(out, eq->bank, eq->lterm, fullterms);
          PRINT_HO_PAREN(out, ')');
@@ -1076,6 +1174,7 @@ void EqnFOFPrint(FILE* out, Eqn_p eq, bool negated,  bool fullterms, bool pcl)
          PRINT_HO_PAREN(out, '(');
          TBPrintTerm(out, eq->bank, eq->rterm, fullterms);
          PRINT_HO_PAREN(out, ')');
+         PRINT_HO_PAREN(out, ')');
       }
       else
       {
@@ -1083,7 +1182,9 @@ void EqnFOFPrint(FILE* out, Eqn_p eq, bool negated,  bool fullterms, bool pcl)
          {
             fputc('~', out);
          }
+         PRINT_HO_PAREN(out, '(');
          TBPrintTerm(out, eq->bank, eq->lterm, fullterms);
+         PRINT_HO_PAREN(out, ')');
       }
    }
    else
@@ -1258,13 +1359,16 @@ Eqn_p EqnCopy(Eqn_p eq, TB_p bank)
    Eqn_p  handle;
    Term_p lterm, rterm;
 
-   lterm = TBInsertNoProps(bank, eq->lterm, DEREF_ALWAYS);
-   rterm = TBInsertNoProps(bank, eq->rterm, DEREF_ALWAYS);
+   lterm = TBInsertNoPropsCached(bank, eq->lterm, DEREF_ALWAYS);
+   rterm = TBInsertNoPropsCached(bank, eq->rterm, DEREF_ALWAYS);
 
-   handle = EqnAlloc(lterm, rterm, bank, false); /* Properties will be
-                                                    taken care of
-                                                    later! */
-   handle->properties = eq->properties;
+   /* Properties will be taken care of later! */
+   handle = EqnAlloc(lterm, rterm, bank, EqnIsPositive(eq));
+   // in HO version positivity is computed in relatively complex way,
+   // thus EPIsPositive must be preserved!
+   handle->properties =
+      (handle->properties & EPIsPositive)
+      | (eq->properties & ~EPIsPositive);
    if(!EqnIsOriented(handle))
    {
       EqnDelProp(handle, EPMaxIsUpToDate);
@@ -1293,10 +1397,13 @@ Eqn_p EqnFlatCopy(Eqn_p eq)
    lterm = eq->lterm;
    rterm = eq->rterm;
 
-   handle = EqnAlloc(lterm, rterm, eq->bank, false); /* Properties will be
-                                                        taken care of
-                                                        later! */
-   handle->properties = eq->properties;
+   /* Properties will be taken care of later! */
+   handle = EqnAlloc(lterm, rterm, eq->bank, EqnIsPositive(eq));
+   // in HO version positivity is computed in relatively complex way,
+   // thus EPIsPositive must be preserved!
+   handle->properties =
+      (handle->properties & EPIsPositive)
+      | (eq->properties & ~EPIsPositive);
    if(!EqnIsOriented(handle))
    {
       EqnDelProp(handle, EPMaxIsUpToDate);
@@ -1325,10 +1432,13 @@ Eqn_p EqnCopyRepl(Eqn_p eq, TB_p bank, Term_p old, Term_p repl)
    lterm = TBInsertRepl(bank, eq->lterm, DEREF_ALWAYS, old, repl);
    rterm = TBInsertRepl(bank, eq->rterm, DEREF_ALWAYS, old, repl);
 
-   handle = EqnAlloc(lterm, rterm, bank, false); /* Properties will be
-                                                    taken care of
-                                                    later! */
-   handle->properties = eq->properties;
+   /* Properties will be taken care of later! */
+   handle = EqnAlloc(lterm, rterm, bank, EqnIsPositive(eq));
+   // in HO version positivity is computed in relatively complex way,
+   // thus EPIsPositive must be preserved!
+   handle->properties =
+      (handle->properties & EPIsPositive)
+      | (eq->properties & ~EPIsPositive);
    EqnDelProp(handle, EPMaxIsUpToDate);
    EqnDelProp(handle, EPIsOriented);
 
@@ -1357,10 +1467,13 @@ Eqn_p EqnCopyReplPlain(Eqn_p eq, TB_p bank, Term_p old, Term_p repl)
    lterm = TBInsertReplPlain(bank, eq->lterm, old, repl);
    rterm = TBInsertReplPlain(bank, eq->rterm, old, repl);
 
-   handle = EqnAlloc(lterm, rterm, bank, false); /* Properties will be
-                                                    taken care of
-                                                    later! */
-   handle->properties = eq->properties;
+   /* Properties will be taken care of later! */
+   handle = EqnAlloc(lterm, rterm, bank, EqnIsPositive(eq));
+   // in HO version positivity is computed in relatively complex way,
+   // thus EPIsPositive must be preserved!
+   handle->properties =
+      (handle->properties & EPIsPositive)
+      | (eq->properties & ~EPIsPositive);
    EqnDelProp(handle, EPMaxIsUpToDate);
    EqnDelProp(handle, EPIsOriented);
 
@@ -1390,10 +1503,13 @@ Eqn_p EqnCopyOpt(Eqn_p eq)
    lterm = TBInsertOpt(eq->bank, eq->lterm, DEREF_ALWAYS);
    rterm = TBInsertOpt(eq->bank, eq->rterm, DEREF_ALWAYS);
 
-   handle = EqnAlloc(lterm, rterm, eq->bank, false); /* Properties will be
-                                                        taken care of
-                                                        later! */
-   handle->properties = eq->properties;
+   /* Properties will be taken care of later! */
+   handle = EqnAlloc(lterm, rterm, eq->bank, EqnIsPositive(eq));
+   // in HO version positivity is computed in relatively complex way,
+   // thus EPIsPositive must be preserved!
+   handle->properties =
+      (handle->properties & EPIsPositive)
+      | (eq->properties & ~EPIsPositive);
    EqnDelProp(handle, EPMaxIsUpToDate);
    EqnDelProp(handle, EPIsOriented);
 
@@ -1422,10 +1538,13 @@ Eqn_p EqnCopyDisjoint(Eqn_p eq)
    lterm = TBInsertDisjoint(eq->bank, eq->lterm);
    rterm = TBInsertDisjoint(eq->bank, eq->rterm);
 
-   handle = EqnAlloc(lterm, rterm, eq->bank, false); /* Properties will be
-                                                        taken care of
-                                                        later! */
-   handle->properties = eq->properties;
+   /* Properties will be taken care of later! */
+   handle = EqnAlloc(lterm, rterm, eq->bank, EqnIsPositive(eq));
+   // in HO version positivity is computed in relatively complex way,
+   // thus EPIsPositive must be preserved!
+   handle->properties =
+      (handle->properties & EPIsPositive)
+      | (eq->properties & ~EPIsPositive);
 
    return handle;
 }
@@ -1864,10 +1983,12 @@ long EqnStructWeightLexCompare(Eqn_p l1, Eqn_p l2)
 
 /*-----------------------------------------------------------------------
 //
-// Function: EqnEqual()
+// Function: EqnEqualDeref()
 //
 //   Test wether two equations are equivalent (modulo commutativity).
-//   Treats equations as _unsigned_ term sets.
+//   Treats equations as _unsigned_ term sets. Follows variable binding
+//   pointers as denoted by d1 and d2.
+//
 //
 // Global Variables: -
 //
@@ -1875,18 +1996,18 @@ long EqnStructWeightLexCompare(Eqn_p l1, Eqn_p l2)
 //
 /----------------------------------------------------------------------*/
 
-bool EqnEqual(Eqn_p eq1,  Eqn_p eq2)
+bool EqnEqualDeref(Eqn_p eq1,  Eqn_p eq2, DerefType d1, DerefType d2)
 {
    bool res;
 
-   res = EqnEqualDirected(eq1, eq2);
+   res = EqnEqualDirectedDeref(eq1, eq2, d1, d2);
 
    if(res || (EqnIsOriented(eq1) && EqnIsOriented(eq2)))
    {
       return res;
    }
    EqnSwapSidesSimple(eq2);
-   res = EqnEqualDirected(eq1, eq2);
+   res = EqnEqualDirectedDeref(eq1, eq2, d1, d2);
    EqnSwapSidesSimple(eq2);
    return res;
 }
@@ -2374,7 +2495,8 @@ CompareResult tfo_literal_compare(OCB_p ocb, Eqn_p eq1, Eqn_p eq2)
       {
          return to_lesser;
       }
-      if(!EqnIsEquLit(eq1)) /* both non-equational! */
+      if(!EqnIsEquLit(eq1) && !TermIsTopLevelFreeVar(eq1->lterm)
+                           && !TermIsTopLevelFreeVar(eq2->lterm)) /* both non-equational! */
       {
          return OCBFunCompare(ocb, eq1->lterm->f_code, eq2->lterm->f_code);
       }
@@ -2389,7 +2511,8 @@ CompareResult tfo_literal_compare(OCB_p ocb, Eqn_p eq1, Eqn_p eq2)
       {
          return to_greater;
       }
-      if(!EqnIsEquLit(eq1)) /* both non-equational! */
+      if(!EqnIsEquLit(eq1) && !TermIsTopLevelFreeVar(eq1->lterm)
+                           && !TermIsTopLevelFreeVar(eq2->lterm)) /* both non-equational! */
       {
          return OCBFunCompare(ocb, eq1->lterm->f_code, eq2->lterm->f_code);
       }
@@ -2594,8 +2717,9 @@ double EqnWeight(Eqn_p eq, double max_multiplier, long vweight, long
 //
 /----------------------------------------------------------------------*/
 
-double  EqnDAGWeight(Eqn_p eq, double max_multiplier, long vweight, long
-                     fweight, long dup_weight, bool new_eqn, bool new_terms)
+double  EqnDAGWeight(Eqn_p eq, double uniqmax_multiplier,
+                     double max_multiplier, long vweight, long fweight,
+                     long dup_weight, bool new_eqn, bool new_terms)
 {
    double res;
    long lweight, rweight;
@@ -2614,13 +2738,48 @@ double  EqnDAGWeight(Eqn_p eq, double max_multiplier, long vweight, long
 
    if(EqnIsOriented(eq))
    {
-      res = (double)rweight;
+      res = uniqmax_multiplier*max_multiplier*(double)lweight;
+      res += (double)rweight;
    }
    else
    {
-      res = (double)rweight*max_multiplier;
+      res = max_multiplier*(double)lweight;
+      res += max_multiplier*(double)rweight;
    }
-   res += (double)lweight*max_multiplier;
+   return res;
+}
+
+/*-----------------------------------------------------------------------
+//
+// Function: EqnDAGWeight2()
+//
+//   Alternative DAG weight of an equation, inspired by Twee
+//   (Smallbone:CADE-202, but details via personal email): Terms are
+//   treated as individual DAGs, the bigger weight of both terms is
+//   boosted by a multiplier. Term order is not considered.
+//
+// Global Variables: -
+//
+// Side Effects    : Sets TPOpFlag
+//
+/----------------------------------------------------------------------*/
+
+double EqnDAGWeight2(Eqn_p eq, double maxw_multiplier,
+                     long vweight, long fweight, long dup_weight)
+{
+   double res;
+   long lweight, rweight;
+
+   lweight = TermDAGWeight(eq->lterm, fweight, vweight, dup_weight, true);
+   rweight = TermDAGWeight(eq->rterm, fweight, vweight, dup_weight, true);
+   //printf("(%ld/%ld)\n", lweight, rweight);
+
+   if(rweight > lweight)
+   {
+      SWAP(lweight, rweight);
+   }
+   res = maxw_multiplier*lweight+rweight;
+
    return res;
 }
 
@@ -3239,7 +3398,66 @@ long EqnCollectSubterms(Eqn_p eqn, PStack_p collector)
 bool EqnHasAppVar(Eqn_p eq)
 {
    assert(eq && eq->lterm && eq->rterm);
-   return TermIsAppliedVar(eq->lterm) || TermIsAppliedVar(eq->rterm);
+   return TermIsAppliedFreeVar(eq->lterm) || TermIsAppliedFreeVar(eq->rterm);
+}
+
+/*-----------------------------------------------------------------------
+//
+// Function: EqnListMapTerms()
+//
+//   Map all terms in the equation list using f.
+//
+// Global Variables: -
+//
+// Side Effects    : Sets the OpFlag of newly collected terms.
+//
+/----------------------------------------------------------------------*/
+
+void EqnMap(Eqn_p lit, TermMapper_p f, void* arg)
+{
+   Term_p lterm = f(arg, lit->lterm);
+   Term_p rterm = f(arg, lit->rterm);
+   DBGTermCheckUnownedSubterm(stdout, lterm, "EqnMapL1");
+   DBGTermCheckUnownedSubterm(stdout, rterm, "EqnMapL2");
+   bool negate = false;
+   TB_p bank = lit->bank;
+
+   if(lterm == bank->false_term)
+   {
+      lterm = bank->true_term;
+      negate = !negate;
+   }
+   if(rterm == bank->false_term)
+   {
+      rterm = bank->true_term;
+      negate = !negate;
+   }
+   if(lterm == bank->true_term)
+   {
+      SWAP(lterm, rterm);
+   }
+   if(rterm != bank->true_term)
+   {
+      EqnSetProp(lit, EPIsEquLiteral);
+   }
+   else
+   {
+      EqnDelProp(lit, EPIsEquLiteral);
+   }
+
+   if(negate)
+   {
+      EqnFlipProp(lit, EPIsPositive);
+   }
+
+   if(lterm != lit->lterm)
+   {
+      EqnDelProp(lit, EPMaxIsUpToDate);
+      EqnDelProp(lit, EPIsOriented);
+   }
+
+   lit->lterm = lterm;
+   lit->rterm = rterm;
 }
 
 /*---------------------------------------------------------------------*/
