@@ -68,17 +68,17 @@ long FormulaSetPreprocConjectures(FormulaSet_p set,
 
 bool WFormulaSimplify(WFormula_p form, TB_p terms);
 
-long WFormulaCNF(WFormula_p form, ClauseSet_p set,
-                 TB_p terms, VarBank_p fresh_vars);
+/* long WFormulaCNF(WFormula_p form, ClauseSet_p set, */
+/*                  TB_p terms, VarBank_p fresh_vars); */
 long WFormulaCNF2(WFormula_p form, ClauseSet_p set,
                   TB_p terms, VarBank_p fresh_vars,
                   long miniscope_limit,
                   bool fool_unroll);
 long FormulaSetSimplify(FormulaSet_p set, TB_p terms, bool gc);
 
-long FormulaSetCNF(FormulaSet_p set, FormulaSet_p archive,
-                   ClauseSet_p clauseset, TB_p terms,
-                   VarBank_p fresh_vars,long def_limit);
+/* long FormulaSetCNF(FormulaSet_p set, FormulaSet_p archive, */
+/*                    ClauseSet_p clauseset, TB_p terms, */
+/*                    VarBank_p fresh_vars,long def_limit); */
 long FormulaSetCNF2(FormulaSet_p set, FormulaSet_p archive,
                     ClauseSet_p clauseset, TB_p terms,
                     VarBank_p fresh_vars,
