@@ -19,6 +19,7 @@
   -----------------------------------------------------------------------*/
 
 #include "cco_batch_spec.h"
+#include "cco_gproc_ctrl.h"
 
 
 
@@ -29,7 +30,7 @@
 char* BatchFilters[] =
 {
    "threshold010000",
-   "gf600_h_gu_R05_F100_L20000"  ,   /* protokoll_X----_auto_sine17 */
+   "gf600_h_gu_R05_F100_L20000"  ,   /* Protokoll_X----_auto_sine17 */
    "gf120_h_gu_R02_F100_L20000"  ,   /* protokoll_X----_auto_sine13 */
    "gf200_gu_RUU_F100_L20000"    ,   /* protokoll_X----_auto_sine08 */
    "gf200_h_gu_R03_F100_L20000"  ,   /* protokoll_X----_auto_sine16 */
@@ -151,19 +152,20 @@ EPCtrl_p batch_create_runner(StructFOFSpec_p ctrl,
                            ax_filter,
                            cspec,
                            fspec);
-   /* fprintf(GlobalOut, "# Spec has %d clauses and %d formulas (%lld)\n",
-      PStackGetSP(cspec), PStackGetSP(fspec), GetSecTimeMod()); */
+   fprintf(GlobalOut, "# Spec has %ld clauses and %ld formulas (%lld)\n",
+           PStackGetSP(cspec), PStackGetSP(fspec), GetSecTimeMod());
 
    file = TempFileName();
    fp   = SecureFOpen(file, "w");
 
-   SigPrintTypeDeclsTSTP(fp, ctrl->sig);
+   SigPrintTypeDeclsTSTP(fp, ctrl->terms->sig);
    PStackClausePrintTSTP(fp, cspec);
    PStackFormulaPrintTSTP(fp, fspec);
    SecureFClose(fp);
-
-   /* fprintf(GlobalOut, "# Written new problem (%lld)\n",
-    * GetSecTimeMod()); */
+   //printf("# ====== Writing filtered file===========\n");
+   //FilePrint(stdout, file);
+   //printf("# =======Filtered file written===========\n");
+   //fprintf(GlobalOut, "# Written new problem (%lld)\n", GetSecTimeMod());
 
    AxFilterPrintBuf(name, 320, ax_filter);
    pctrl = ECtrlCreateGeneric(executable, name, options, extra_options, cpu_time, file);
@@ -567,7 +569,7 @@ long BatchStructFOFSpecInit(BatchSpec_p spec,
    long res;
 
    res = StructFOFSpecParseAxioms(ctrl, spec->includes, spec->format, default_dir);
-   StructFOFSpecInitDistrib(ctrl);
+   StructFOFSpecInitDistrib(ctrl, false);
 
    return res;
 }
@@ -589,14 +591,17 @@ long BatchStructFOFSpecInit(BatchSpec_p spec,
 
 void StructFOFSpecAddProblem(StructFOFSpec_p ctrl,
                              ClauseSet_p clauses,
-                             FormulaSet_p formulas)
+                             FormulaSet_p formulas,
+                             bool trim)
 {
-   GenDistribSizeAdjust(ctrl->f_distrib, ctrl->sig);
+   GenDistribSizeAdjust(ctrl->f_distrib, ctrl->terms->sig);
+   TBGCRegisterClauseSet(ctrl->terms, clauses);
    PStackPushP(ctrl->clause_sets, clauses);
+   TBGCRegisterFormulaSet(ctrl->terms, formulas);
    PStackPushP(ctrl->formula_sets, formulas);
 
    GenDistribAddClauseSet(ctrl->f_distrib, clauses, 1);
-   GenDistribAddFormulaSet(ctrl->f_distrib, formulas, 1);
+   GenDistribAddFormulaSet(ctrl->f_distrib, formulas, 1, trim);
 }
 
 
@@ -605,7 +610,8 @@ void StructFOFSpecAddProblem(StructFOFSpec_p ctrl,
 // Function: StructFOFSpecBacktrackToSpec()
 //
 //   Backtrack the state to the spec state, i.e. backtrack the
-//   frequency count and free the extra clause sets.
+//   frequency count and free the extra clause sets. Also backtracks
+//   the signature to forget all new symbols.
 //
 // Global Variables: -
 //
@@ -627,10 +633,16 @@ void StructFOFSpecBacktrackToSpec(StructFOFSpec_p ctrl)
    while(PStackGetSP(ctrl->clause_sets)>ctrl->shared_ax_sp)
    {
       clauses = PStackPopP(ctrl->clause_sets);
+      GCDeregisterClauseSet(ctrl->terms->gc, clauses);
       ClauseSetFree(clauses);
       formulas = PStackPopP(ctrl->formula_sets);
+      GCDeregisterFormulaSet(ctrl->terms->gc, formulas);
       FormulaSetFree(formulas);
    }
+   TBGCCollect(ctrl->terms);
+   SigBacktrack(ctrl->terms->sig, ctrl->shared_ax_f_count);
+
+   problemType = PROBLEM_NOT_INIT;
 }
 
 
@@ -672,6 +684,12 @@ long StructFOFSpecGetProblem(StructFOFSpec_p ctrl,
                                res_clauses,
                                res_formulas);
          break;
+   case AFLambdaDefines:
+         res = SelectDefinitions(ctrl->clause_sets,
+                                ctrl->formula_sets,
+                                res_clauses,
+                                res_formulas);
+         break;
    default:
          assert(false && "Unknown AxFilter type");
          break;
@@ -718,7 +736,8 @@ bool BatchProcessProblem(BatchSpec_p spec,
 
    StructFOFSpecAddProblem(ctrl,
                            cset,
-                           fset);
+                           fset,
+                           false);
 
    start = GetSecTime();
    end   = start+wct_limit;
@@ -732,11 +751,11 @@ bool BatchProcessProblem(BatchSpec_p spec,
       {
          used = now-start;
          handle = batch_create_runner(ctrl, spec->executable,
-                                      BatchStrategiesDiv[i],
+                                      BatchStrategies[i],
                                       answers,
                                       MIN((wct_limit+1)/2, wct_limit-used),
                                       AxFilterSetFindFilter(filters,
-                                                            BatchFiltersDiv[i]));
+                                                            BatchFilters[i]));
          EPCtrlSetAddProc(procs, handle);
          i++;
       }
@@ -798,6 +817,7 @@ bool BatchProcessProblem(BatchSpec_p spec,
 
    StructFOFSpecBacktrackToSpec(ctrl);
    /* cset and fset are freed in Backtrack */
+
    AxFilterSetFree(filters);
    EPCtrlSetFree(procs, true);
 
@@ -832,12 +852,13 @@ bool BatchProcessFile(BatchSpec_p spec,
    FormulaSet_p fset;
    FILE* fp;
 
-
-   fprintf(GlobalOut, "\n# Processing %s -> %s\n", source, dest);
-   fprintf(GlobalOut, "# SZS status Started for %s\n", source);
-   fflush(GlobalOut);
+   //fprintf(GlobalOut, "\n# Processing %s -> %s\n", source, dest);
+   //fprintf(GlobalOut, "# SZS status Started for %s\n", source);
+   //fflush(GlobalOut);
 
    in = CreateScanner(StreamTypeFile, source, true, default_dir, true);
+   //printf("# Scanner for '%s' created\n", source);
+   fflush(stdout);
    ScannerSetFormat(in, TSTPFormat);
 
    dummy = ClauseSetAlloc();
@@ -862,8 +883,8 @@ bool BatchProcessFile(BatchSpec_p spec,
                              false);
    SecureFClose(fp);
 
-   fprintf(GlobalOut, "# SZS status Ended for %s\n\n", source);
-   fflush(GlobalOut);
+   //fprintf(GlobalOut, "# SZS status Ended for %s\n\n", source);
+   //fflush(GlobalOut);
 
    return res;
 }
@@ -1102,14 +1123,22 @@ void BatchProcessVariants(BatchSpec_p spec, char* variants[], char* provers[],
               prob_count-solved_count, var_count-variant, concrete_prob_count);
 
       // Loading axioms here!
-      ctrl = StructFOFSpecAlloc();
-      concrete_batch_struct_FOF_spec_init(spec,
-                                         ctrl,
-                                         default_dir,
-                                         variants[variant]);
+      // DISABLED FOR CASC-28 - no shared axioms. Inconsisten Spec!
+      //ctrl = StructFOFSpecAlloc();
+      //concrete_batch_struct_FOF_spec_init(spec,
+      //                                   ctrl,
+      //                                   default_dir,
+      //                                  variants[variant]);
 
       for(i=0; i<PStackGetSP(spec->source_files); i++)
       {
+         // CASC-28/J10-Hack
+         ctrl = StructFOFSpecAlloc();
+         concrete_batch_struct_FOF_spec_init(spec,
+                                             ctrl,
+                                             default_dir,
+                                             variants[variant]);
+         // CASC-28/J10-Hack ends
          abstract_name = PStackElementP(spec->source_files, i);
          if(PDArrayElementInt(solved, i))
          {
@@ -1134,26 +1163,58 @@ void BatchProcessVariants(BatchSpec_p spec, char* variants[], char* provers[],
             }
             DStrAppendStr(dest_name, PStackElementP(spec->dest_files, i));
 
-            success = BatchProcessFile(spec, per_prob_time,
-                                       ctrl, default_dir,
-                                       concrete_name,
-                                       DStrView(dest_name));
+
+            fprintf(GlobalOut, "\n# Processing %s -> %s\n",
+                    concrete_name, DStrView(dest_name));
+            fprintf(GlobalOut, "# SZS status Started for %s\n", concrete_name);
+            fflush(GlobalOut);
+
+
+            EGPCtrl_p handle = EGPCtrlCreate("E-LTB wrapper", 1, 1000000);
+            char buffer[EGPCTRL_BUFSIZE];
+            if(!handle)
+            {
+               // It's the wrapped child, do work
+               success = BatchProcessFile(spec, per_prob_time,
+                                          ctrl, default_dir,
+                                          concrete_name,
+                                          DStrView(dest_name));
+               exit(success);
+            }
+            // else
+            while(!EGPCtrlGetResult(handle, buffer,EGPCTRL_BUFSIZE))
+            {
+               // Nothing
+            }
+            success = (handle->result==PRTheorem)||(handle->result==PRUnsatisfiable);
+            fprintf(GlobalOut, "%s", DStrView(handle->output));
+            EGPCtrlFree(handle);
+
             if(success)
             {
                solved_count++;
                PDArrayAssignInt(solved, i, 1);
                concrete_prob_count -= (var_count-variant);
+               //printf("# SUCCESS: %ld abstract solved, %ld concrete remaining\n",
+               //solved_count, concrete_prob_count);
             }
             else
             {
                concrete_prob_count--;
+               //printf("# FAILURE: %ld abstract solved, %ld concrete remaining\n",
+               //solved_count, concrete_prob_count);
             }
+            fprintf(GlobalOut, "# SZS status Ended for %s\n\n", concrete_name);
+            fflush(GlobalOut);
 
             FREE(concrete_name);
          }
+         // CASC-28-Hack
+         StructFOFSpecFree(ctrl);
       }
       spec->executable = save_exec;
-      StructFOFSpecFree(ctrl);
+      // See above - disabled for CASC-28
+      //StructFOFSpecFree(ctrl);
    }
    DStrFree(dest_name);
    PDArrayFree(solved);
