@@ -28,7 +28,7 @@
 
 char* help_message = "\
 # Note : Block commands that are of the form of \"COMMAND <NAME> ... GO\"\n\
-# should have the \"COMMAND <NAME>\" and GO each on a seperate line of\n\
+# should have the \"COMMAND <NAME>\" and GO each on a separate line of\n\
 # their own. The block should be in between these two.\n\
 #\n\
 #- ADD <NAME> ... GO : Uploads a new axiom set with the name <NAME>.\n\
@@ -96,7 +96,7 @@ char* add_command(InteractiveSpec_p interactive,
 char* stage_command(InteractiveSpec_p interactive, DStr_p axiom_set);
 char* list_command(InteractiveSpec_p interactive);
 void  quit_command(InteractiveSpec_p interactive);
-char* remove_commad(InteractiveSpec_p interactive, DStr_p axiom_set);
+char* remove_command(InteractiveSpec_p interactive, DStr_p axiom_set);
 char* download_command(InteractiveSpec_p interactive, DStr_p axiom_set);
 char* unstage_command(InteractiveSpec_p interactive, DStr_p axiom_set);
 char* load_command(InteractiveSpec_p interactive, DStr_p filename);
@@ -284,7 +284,7 @@ char* stage_command(InteractiveSpec_p interactive, DStr_p axiom_set)
          }
          else
          {
-            StructFOFSpecAddProblem(interactive->ctrl, handle->cset, handle->fset);
+            StructFOFSpecAddProblem(interactive->ctrl, handle->cset, handle->fset, false);
             handle->staged = 1;
             interactive->ctrl->shared_ax_sp = PStackGetSP(interactive->ctrl->clause_sets);
             return OK_STAGED_MESSAGE;
@@ -438,7 +438,7 @@ void quit_command(InteractiveSpec_p interactive)
 //
 /----------------------------------------------------------------------*/
 
-char* remove_commad(InteractiveSpec_p interactive, DStr_p axiom_set)
+char* remove_command(InteractiveSpec_p interactive, DStr_p axiom_set)
 {
    AxiomSet_p    handle;
    PStack_p spare_stack;
@@ -573,7 +573,7 @@ char* unstage_command(InteractiveSpec_p interactive, DStr_p axiom_set)
 
       if(strcmp( DStrView(axiom_set), DStrView(chandle->identifier)) == 0 )
       {
-         GenDistribAddFormulaSet(interactive->ctrl->f_distrib, fhandle, -1);
+         GenDistribAddFormulaSet(interactive->ctrl->f_distrib, fhandle, false, -1);
          GenDistribAddClauseSet(interactive->ctrl->f_distrib, chandle, -1);
          found = 1;
          break;
@@ -991,7 +991,7 @@ void StartDeductionServer(BatchSpec_p spec,
          AcceptInpId(in, REMOVE_COMMAND);
          DStrReset(dummyStr);
          AcceptAxiomSetName(in, dummyStr);
-         print_to_outstream(remove_commad(interactive, dummyStr), fp, sock_fd);
+         print_to_outstream(remove_command(interactive, dummyStr), fp, sock_fd);
       }
       else if(TestInpId(in, DOWNLOAD_COMMAND))
       {
