@@ -22,6 +22,7 @@ Changes
 -----------------------------------------------------------------------*/
 
 #include "che_to_autoselect.h"
+#include "che_new_autoschedule.h"
 
 
 
@@ -63,7 +64,10 @@ void init_oparms(OrderParms_p oparms)
    oparms->conj_axiom_mod  = 0;
    oparms->axiom_only_mod  = 0;
    oparms->lit_cmp         = LCNormal;
-
+   oparms->ho_order_kind   = LFHO_ORDER;
+   oparms->db_w = DEFAULT_DB_WEIGHT;
+   oparms->lam_w = DEFAULT_LAMBDA_WEIGHT;
+   oparms->force_kbo_var_weight = false;
 }
 
 /*-----------------------------------------------------------------------
@@ -118,14 +122,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_auto_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_auto_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = order_kind;
    OUTPRINT(1, "\n# Auto-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -156,7 +161,7 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autocasc_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autocasc_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = SpecLimitsAlloc();
@@ -183,9 +188,10 @@ OCB_p generate_autocasc_ordering(ProofState_p state, SpecFeature_p spec)
    oparms.to_weight_gen   = WSelectMaximal;
    oparms.to_prec_gen     = PUnaryFirst;
    oparms.lit_cmp         = LCNormal;
+   oparms.ho_order_kind  = order_kind;
    OUTPRINT(1, "\n# Auto-Ordering is analysing problem.\n");
 
-#include "che_auto_cases.c"
+
 
    if(OutputLevel)
    {
@@ -229,7 +235,7 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autodev_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autodev_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
@@ -239,9 +245,10 @@ OCB_p generate_autodev_ordering(ProofState_p state, SpecFeature_p spec)
    oparms.to_weight_gen   = WSelectMaximal;
    oparms.to_prec_gen     = PUnaryFirst;
    oparms.lit_cmp         = LCNormal;
+   oparms.ho_order_kind   = order_kind;
 
    OUTPRINT(1, "\n# Auto-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
 
    if(OutputLevel)
    {
@@ -292,14 +299,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched0_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched0_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched0-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -315,14 +323,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched1_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched1_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched1-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -338,14 +347,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched2_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched2_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched2-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -361,14 +371,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched3_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched3_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched3-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -384,14 +395,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched4_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched4_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched4-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -407,14 +419,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched5_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched5_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched5-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -430,14 +443,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched6_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched6_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched6-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -453,14 +467,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched7_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched7_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched7-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -477,14 +492,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched8_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched8_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched8-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -500,14 +516,15 @@ GCC_DIAGNOSTIC_PUSH
 __attribute__((optnone))
 __attribute__((optimize(0)))
 #endif
-OCB_p generate_autosched9_ordering(ProofState_p state, SpecFeature_p spec)
+OCB_p generate_autosched9_ordering(ProofState_p state, SpecFeature_p spec, HoOrderKind ho_order_kind)
 {
    OrderParmsCell  oparms;
    SpecLimits_p    limits = CreateDefaultSpecLimits();
 
    init_oparms(&oparms);
+   oparms.ho_order_kind = ho_order_kind;
    OUTPRINT(1, "\n# AutoSched9-Ordering is analysing problem.\n");
-#include "che_auto_cases.c"
+
    print_oparms(&oparms);
    SpecLimitsCellFree(limits);
    return TOCreateOrdering(state, &oparms, NULL, NULL);
@@ -846,58 +863,6 @@ OCB_p TOSelectOrdering(ProofState_p state, HeuristicParms_p params,
 
       result = OrderFindOptimal(&local, OrderEvaluate, state, params);
    }
-   else if(tmp.ordertype == AUTO)
-   {
-      result = generate_auto_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOCASC)
-   {
-      result = generate_autocasc_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTODEV)
-   {
-      result = generate_autodev_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED0)
-   {
-      result = generate_autosched0_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED1)
-   {
-      result = generate_autosched1_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED2)
-   {
-      result = generate_autosched2_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED3)
-   {
-      result = generate_autosched3_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED4)
-   {
-      result = generate_autosched4_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED5)
-   {
-      result = generate_autosched5_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED6)
-   {
-      result = generate_autosched6_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED7)
-   {
-      result = generate_autosched7_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED8)
-   {
-      result = generate_autosched8_ordering(state, specs);
-   }
-   else if(tmp.ordertype == AUTOSCHED9)
-   {
-      result = generate_autosched9_ordering(state, specs);
-   }
    else
    {
       if(tmp.ordertype == NoOrdering)
@@ -934,42 +899,43 @@ OCB_p  TOCreateOrdering(ProofState_p state, OrderParms_p params,
 {
    OCB_p handle;
    bool prec_by_weight = pre_precedence?false:true;
+   assert(params->ho_order_kind == LFHO_ORDER || params->ho_order_kind == LAMBDA_ORDER);
 
    /* printf("TOCreateOrdering(%d, %d, %d, %ld)\n", params->ordertype,
       params->to_weight_gen, params->to_prec_gen,
       params->to_const_weight); */
 
-#ifdef ENABLE_LFHO
-   if(problemType == PROBLEM_HO)
-   {
-      params->ordertype = KBO6;
-   }
-#endif
+// #ifdef ENABLE_LFHO
+//    if(problemType == PROBLEM_HO)
+//    {
+//       params->ordertype = KBO6;
+//    }
+// #endif
 
    switch(params->ordertype)
    {
    case LPO:
-         handle = OCBAlloc(LPO, prec_by_weight, state->signature);
+         handle = OCBAlloc(LPO, prec_by_weight, state->signature, params->ho_order_kind);
          TOGeneratePrecedence(handle, state->axioms, pre_precedence,
                               params);
          break;
    case LPOCopy:
-         handle = OCBAlloc(LPOCopy, prec_by_weight, state->signature);
+         handle = OCBAlloc(LPOCopy, prec_by_weight, state->signature, params->ho_order_kind);
          TOGeneratePrecedence(handle, state->axioms, pre_precedence,
                               params);
          break;
    case LPO4:
-         handle = OCBAlloc(LPO4, prec_by_weight, state->signature);
+         handle = OCBAlloc(LPO4, prec_by_weight, state->signature, params->ho_order_kind);
          TOGeneratePrecedence(handle, state->axioms, pre_precedence,
                               params);
          break;
    case LPO4Copy:
-         handle = OCBAlloc(LPO4Copy, prec_by_weight, state->signature);
+         handle = OCBAlloc(LPO4Copy, prec_by_weight, state->signature, params->ho_order_kind);
          TOGeneratePrecedence(handle, state->axioms, pre_precedence,
                               params);
          break;
    case KBO:
-         handle = OCBAlloc(KBO, prec_by_weight, state->signature);
+         handle = OCBAlloc(KBO, prec_by_weight, state->signature, params->ho_order_kind);
          TOGeneratePrecedence(handle, state->axioms, pre_precedence,
                               params);
          TOGenerateWeights(handle, state->axioms,
@@ -977,7 +943,7 @@ OCB_p  TOCreateOrdering(ProofState_p state, OrderParms_p params,
                            params);
          break;
    case KBO6:
-         handle = OCBAlloc(KBO6, prec_by_weight, state->signature);
+         handle = OCBAlloc(KBO6, prec_by_weight, state->signature, params->ho_order_kind);
          TOGeneratePrecedence(handle, state->axioms, pre_precedence,
                               params);
          TOGenerateWeights(handle, state->axioms,
