@@ -65,6 +65,7 @@ typedef enum
    OPT_CPU_LIMIT,
    OPT_SOFTCPU_LIMIT,
    OPT_RUSAGE_INFO,
+   OPT_SELECT_STRATEGY,
    OPT_PRINT_STRATEGY,
    OPT_PARSE_STRATEGY,
    OPT_STEP_LIMIT,
@@ -190,7 +191,6 @@ typedef enum
    OPT_HEURISTIC,
    OPT_FREE_NUMBERS,
    OPT_FREE_OBJECTS,
-   OPT_DEF_CNF_OLD,
    OPT_DEF_CNF,
    OPT_FOOL_UNROLL,
    OPT_MINISCOPE_LIMIT,
@@ -487,7 +487,7 @@ OptCell opts[] =
     "immediately after reaching the time limit, regardless of internal "
     "state. As a side effect, this option will inhibit core file "
     "writing. Please note that if you use both --cpu-limit and "
-    "--soft-cpu-limit, the soft limit has to"
+    "--soft-cpu-limit, the soft limit has to "
     "be smaller than the hard limit to have any effect. "},
 
    {OPT_SOFTCPU_LIMIT,
@@ -512,13 +512,20 @@ OptCell opts[] =
     "more information with the rusage() system call, you will also "
     "get information about memory consumption."},
 
+   {OPT_SELECT_STRATEGY,
+    '\0', "select-strategy",
+    ReqArg, NULL,
+    "Select one of the built-in strategies and set all proof search "
+    "parameters accordingly."},
+
    {OPT_PRINT_STRATEGY,
     '\0', "print-strategy",
     OptArg, ">current-strategy<",
     "Print a representation of all search parameters and their setting "
     "of a given strategy, then terminate. If no argument is given, "
     "the current strategy is printed. Use the reserved name '>all-strats<'"
-    "to get a description of all built-in strategies."},
+    "to get a description of all built-in strategies,  '>all-names<' "
+    "to get a list of all names of strategies."},
 
    {OPT_PARSE_STRATEGY,
     '\0', "parse-strategy",
@@ -762,7 +769,7 @@ OptCell opts[] =
 
    {OPT_PRESAT_SIMPLIY,
     '\0', "presat-simplify",
-    NoArg, NULL,
+    OptArg, "true",
     "Before proper saturation do a complete interreduction of "
     "the proof state."},
 
@@ -1498,21 +1505,12 @@ OptCell opts[] =
     "subformulae to avoid exponential blow-up. The optional argument "
     "is a fudge factor that determines when definitions are introduced. "
     "0 disables definitions completely. The default works well."},
-    // OPT_FOOL_UNROLL
+
    {OPT_FOOL_UNROLL,
     '\0', "fool-unroll",
     ReqArg, NULL,
     "Enable or disable FOOL unrolling. Useful for some SH problems."},
 
-   {OPT_DEF_CNF_OLD,
-    '\0', "old-cnf",
-    OptArg, TFORM_RENAME_LIMIT_STR,
-    "As the previous option, but use the classical, well-tested "
-    "clausification algorithm as opposed to the newewst one which "
-    "avoides some algorithmic pitfalls and hence works better on "
-    "some exotic formulae. The two may produce slightly different "
-    "(but equisatisfiable) clause normal forms."},
-
    {OPT_MINISCOPE_LIMIT,
     '\0', "miniscope-limit",
     OptArg, TFORM_MINISCOPE_LIMIT_STR,
