@@ -6,7 +6,7 @@ Author: Stephan Schulz (schulz@eprover.org)
 
 Contents
 
-  Define global macro for version number and meta-information.
+  Define global macros for version number and meta-information.
 
   Copyright 2009-2024 by the authors (see DOC/CONTRIBUTORS).
   This code is released under the GNU General Public Licence and
@@ -31,9 +31,9 @@ Contents
 /*                Exported Functions and Variables                     */
 /*---------------------------------------------------------------------*/
 
-/* git tag E-3.2.0 */
+/* git tag E-3.2.5 */
 
-#define PVERSION      "3.2.0"
+#define PVERSION      "3.2.5"
 
 #include "e_gitcommit.h"
 
@@ -57,6 +57,7 @@ Contents
 #define STS_COPYRIGHT "Copyright 1998-2024 by Stephan Schulz"
 #define CTR_COPYRIGHT "and the E contributors (see DOC/CONTRIBUTORS)."
 #define STS_SNAIL "Stephan Schulz\nDHBW Stuttgart\nFakultaet Technik\nInformatik\nLerchenstrasse 1\n70174 Stuttgart\nGermany\n"
+#define HO_MAIL "jasmin.blanchette@gmail.com"
 
 #define E_FOOTER \
 STS_COPYRIGHT", " STS_MAIL ",\n\
@@ -83,6 +84,21 @@ directory of the distribution in the file COPYING); if not, write to\n\
 the Free Software Foundation, Inc., 59 Temple Place, Suite 330,\n\
 Boston, MA  02111-1307 USA\n\
 \n\
+We welcome bug reports and even reasonable questions. If the prover\n\
+behaves in an unexpected way, please include the following\n\
+information:\n\
+\n\
+- What did you observe?\n\
+- What did you expect?\n\
+- The output of `eprover --version`\n\
+- The full commandline that lead to the unexpected behaviour\n\
+- The input file(s) that lead to the unexpected behaviour\n\
+\n\
+Most bug reports should be send to <"STS_MAIL">. Bug reports with \n\
+respect to the HO-version should be send to or at least copied to \n\
+<"HO_MAIL">. Please remember that this is an unpaid\n\
+volunteer service.\n\
+\n\
 The original copyright holder can be contacted via email or as\n\
 \n" STS_SNAIL "\n"
 
