@@ -26,6 +26,7 @@
 #include <ccl_splitting.h>
 #include <ccl_grounding.h>
 #include <che_clausesetfeatures.h>
+#include <che_hcb.h>
 #include <ccl_formulafunc.h>
 #include <e_version.h>
 
@@ -56,7 +57,6 @@ typedef enum
    OPT_TSTP_PARSE,
    OPT_TSTP_PRINT,
    OPT_TSTP_FORMAT,
-   OPT_DEF_CNF_OLD,
    OPT_DEF_CNF,
    OPT_MINISCOPE_LIMIT,
    OPT_DIMACS_PRINT,
@@ -134,7 +134,7 @@ OptCell opts[] =
    {OPT_SUPRESS_RESULT,
     '\0', "suppress-result",
     NoArg, NULL,
-    "Supress actual printing of the result, just give a short message "
+    "Suppress actual printing of the result, just give a short message "
     "about success. Useful mainly for test runs."},
 
    {OPT_LOP_PARSE,
@@ -226,15 +226,6 @@ OptCell opts[] =
     "is a fudge factor that determines when definitions are introduced. "
     "0 disables definitions completely. The default works well."},
 
-   {OPT_DEF_CNF_OLD,
-    '\0', "old-cnf",
-    OptArg, TFORM_RENAME_LIMIT_STR,
-    "As the previous option, but use the classical, well-tested "
-    "clausification algorithm as opposed to the newewst one which "
-    "avoides some algorithmic pitfalls and hence works better on "
-    "some exotic formulae. The two may produce slightly different "
-    "(but equisatisfiable) clause normal forms."},
-
    {OPT_MINISCOPE_LIMIT,
     '\0', "miniscope-limit",
     OptArg, TFORM_MINISCOPE_LIMIT_STR,
@@ -306,9 +297,9 @@ OptCell opts[] =
     'g', "give-up",
     ReqArg, NULL,
     "Give up early if the problem is unlikely to be reasonably"
-    " small. If run without constraints, the programm will give up if"
+    " small. If run without constraints, the program will give up if"
     " the clause with the largest number of instances will be expanded"
-    " into more than this number of instances. If run with contraints,"
+    " into more than this number of instances. If run with constraints,"
     " the program keeps a running count and will terminate if the"
     " estimated total"
     " number of clauses would exceed this value . A value of 0 will"
@@ -345,8 +336,7 @@ char   *outname = NULL;
 IOFormat parse_format = AutoFormat;
 bool   dimacs_format = false;
 int    split_tries = 0;
-bool   new_cnf          = true,
-       unit_sub = true,
+bool   unit_sub = true,
        unit_res = true,
        taut_check = true,
        add_single_instance = false,
@@ -358,10 +348,10 @@ bool   new_cnf          = true,
        fix_minisat = false,
        app_encode = false;
 long   give_up = 0,
-       miniscope_limit  = 1000,
+       miniscope_limit  = DEFAULT_MINISCOPE_LIMIT,
        initial_literals = 0,
        initial_clauses = 0;
-ProblemType problemType  = PROBLEM_NOT_INIT;
+long FormulaDefLimit = 24;
 
 /*---------------------------------------------------------------------*/
 /*                      Forward Declarations                           */
@@ -377,7 +367,6 @@ void print_help(FILE* out);
 int main(int argc, char* argv[])
 {
    TB_p            terms;
-   GCAdmin_p       collector;
    VarBank_p       freshvars;
    TypeBank_p      type_bank;
    Sig_p           sig;
@@ -416,16 +405,15 @@ int main(int argc, char* argv[])
    sig          = SigAlloc(type_bank);
    SigInsertInternalCodes(sig);
    terms        = TBAlloc(sig);
-   collector    = GCAdminAlloc(terms);
    def_store    = DefStoreAlloc(terms);
    clauses      = ClauseSetAlloc();
    dummy        = ClauseSetAlloc();
    formulas     = FormulaSetAlloc();
    f_ax_archive = FormulaSetAlloc();
 
-   GCRegisterClauseSet(collector, clauses);
-   GCRegisterFormulaSet(collector, formulas);
-   GCRegisterFormulaSet(collector, f_ax_archive);
+   TBGCRegisterClauseSet(terms, clauses);
+   TBGCRegisterFormulaSet(terms, formulas);
+   TBGCRegisterFormulaSet(terms, f_ax_archive);
 
    for(i=0; state->argv[i]; i++)
    {
@@ -450,15 +438,17 @@ int main(int argc, char* argv[])
       VERBOUT("Negated conjectures.\n");
    }
    freshvars = VarBankAlloc(type_bank);
-   if(FormulaSetCNF(formulas, f_ax_archive, clauses, terms, freshvars, collector))
+   if(FormulaSetCNF2(formulas, f_ax_archive, clauses, terms, freshvars,
+                     1048576, FormulaDefLimit,
+                     true, true, true, true))
    {
       VERBOUT("CNFization done\n");
    }
    VarBankFree(freshvars);
 
-   GCDeregisterFormulaSet(collector, formulas);
+   TBGCDeregisterFormulaSet(terms, formulas);
    FormulaSetFree(formulas);
-   GCDeregisterFormulaSet(collector, f_ax_archive);
+   TBGCDeregisterFormulaSet(terms, f_ax_archive);
    FormulaSetFree(f_ax_archive);
 
    ClauseSetRemoveSuperfluousLiterals(clauses);
@@ -478,7 +468,7 @@ int main(int argc, char* argv[])
                             false);
    def_store->def_clauses->fvindex = FVIAnchorAlloc(cspec, perm);
 
-   SpecFeaturesCompute(&features, clauses, sig);
+   SpecFeaturesCompute(&features, clauses, formulas, f_ax_archive, terms);
 
    if(!SpecNoEq(&features))
    {
@@ -611,7 +601,6 @@ int main(int argc, char* argv[])
 #ifndef FAST_EXIT
    GroundSetFree(groundset);
    ClauseSetFree(clauses);
-   GCAdminFree(collector);
 
    terms->sig = NULL;
    DefStoreFree(def_store);
@@ -725,9 +714,6 @@ CLState_p process_options(int argc, char* argv[])
       case OPT_DIMACS_PRINT:
             dimacs_format = true;
             break;
-      case OPT_DEF_CNF_OLD:
-            new_cnf = false;
-            /* Intentional fall-through */
       case OPT_DEF_CNF:
             FormulaDefLimit     = CLStateGetIntArg(handle, arg);
             break;
@@ -825,7 +811,7 @@ CLState_p process_options(int argc, char* argv[])
       if(getrlimit(RLIMIT_CPU, &limit))
       {
          TmpErrno = errno;
-         SysError("Unable to get sytem cpu time limit", SYS_ERROR);
+         SysError("Unable to get system cpu time limit", SYS_ERROR);
       }
       SystemTimeLimit = limit.rlim_max;
       if(SoftTimeLimit!=RLIM_INFINITY)
