@@ -57,6 +57,7 @@ PERF_CTR_DEFINE(SatTimer);
 char              *outname = NULL;
 char              *watchlist_filename = NULL;
 char              *parse_strategy_filename = NULL;
+char              *select_strategy = NULL;
 char              *print_strategy = NULL;
 HeuristicParms_p  h_parms;
 FVIndexParms_p    fvi_parms;
@@ -74,7 +75,6 @@ bool              print_sat = false,
    indexed_subsumption = true,
    syntax_only = false,
    prune_only = false,
-   new_cnf = true,
    cnf_only = false,
    inf_sys_complete = true,
    assume_inf_sys_complete = false,
@@ -95,8 +95,7 @@ long              step_limit = LONG_MAX,
    total_limit = LONG_MAX,
    cores       = 1,
    generated_limit = LONG_MAX,
-   relevance_prune_level = 0,
-   miniscope_limit = 1048576;
+   relevance_prune_level = 0;
 long long tb_insert_limit = LLONG_MAX;
 bool lift_lambdas = true;
 int num_cpus = 1;
@@ -218,6 +217,10 @@ ProofState_p parse_spec(CLState_p state,
    }
    VERBOUT2("Specification read\n");
 
+   ProofStateProcessDistinct(proofstate);
+
+   VERBOUT2("$distinct directives processed\n");
+
    proofstate->has_interpreted_symbols =
       FormulaSetHasInterpretedSymbol(proofstate->f_axioms);
    parsed_ax_no = ProofStateAxNo(proofstate);
@@ -291,12 +294,20 @@ void strategy_io(HeuristicParms_p h_parms, PStack_p hcb_definitions)
       }
       DestroyScanner(in);
    }
+   if(select_strategy)
+   {
+      GetHeuristicWithName(select_strategy, h_parms);
+   }
 
    if(print_strategy)
    {
       if(strcmp(print_strategy, ">all-strats<")==0)
       {
-         StrategiesPrintPredefined(GlobalOut);
+         StrategiesPrintPredefined(GlobalOut, false);
+      }
+      else if(strcmp(print_strategy, ">all-names<")==0)
+      {
+         StrategiesPrintPredefined(GlobalOut, true);
       }
       else
       {
@@ -545,7 +556,7 @@ int main(int argc, char* argv[])
    SpecLimits_p spec_limits = NULL;
    RawSpecFeatureCell raw_features;
    SpecFeatureCell features;
-   int sched_idx;
+   int sched_idx = -1;
    Schedule_p preproc_schedule = NULL;
    rlim_t wc_sched_limit;
    Derivation_p deriv;
@@ -641,29 +652,17 @@ int main(int argc, char* argv[])
    }
 
    VERBOUT("Clausification started.\n");
-   if(new_cnf)
-   {
-      cnf_size = FormulaSetCNF2(proofstate->f_axioms,
-                                proofstate->f_ax_archive,
-                                proofstate->axioms,
-                                proofstate->terms,
-                                proofstate->freshvars,
-                                miniscope_limit,
-                                h_parms->formula_def_limit,
-                                h_parms->lift_lambdas,
-                                h_parms->lambda_to_forall,
-                                h_parms->unroll_only_formulas,
-                                h_parms->fool_unroll);
-   }
-   else
-   {
-      cnf_size = FormulaSetCNF(proofstate->f_axioms,
-                               proofstate->f_ax_archive,
-                               proofstate->axioms,
-                               proofstate->terms,
-                               proofstate->freshvars,
-                               h_parms->formula_def_limit);
-   }
+   cnf_size = FormulaSetCNF2(proofstate->f_axioms,
+                             proofstate->f_ax_archive,
+                             proofstate->axioms,
+                             proofstate->terms,
+                             proofstate->freshvars,
+                             h_parms->miniscope_limit,
+                             h_parms->formula_def_limit,
+                             h_parms->lift_lambdas,
+                             h_parms->lambda_to_forall,
+                             h_parms->unroll_only_formulas,
+                             h_parms->fool_unroll);
    VERBOUT("Clausification done.\n");
 
    if(cnf_size)
@@ -726,7 +725,7 @@ int main(int argc, char* argv[])
                            h_parms, proofstate->terms,
                            proofstate->tmp_terms, proofstate->freshvars);
    }
-   if((strategy_scheduling && sched_idx != -1) || auto_conf)
+   if((strategy_scheduling && sched_idx != -1) || (auto_conf && !cnf_only))
    {
       if(!spec_limits)
       {
@@ -1346,6 +1345,9 @@ CLState_p process_options(int argc, char* argv[])
       case OPT_RUSAGE_INFO:
             print_rusage = true;
             break;
+      case OPT_SELECT_STRATEGY:
+            select_strategy = arg;
+            break;
       case OPT_PRINT_STRATEGY:
             print_strategy = arg;
             break;
@@ -1493,7 +1495,7 @@ CLState_p process_options(int argc, char* argv[])
             relevance_prune_level = CLStateGetIntArg(handle, arg);
             break;
       case OPT_PRESAT_SIMPLIY:
-            h_parms->presat_interreduction = true;
+            h_parms->presat_interreduction = CLStateGetBoolArg(handle, arg);
             break;
       case OPT_AC_HANDLING:
             if(strcmp(arg, "None")==0)
@@ -1835,13 +1837,13 @@ CLState_p process_options(int argc, char* argv[])
             break;
       case OPT_SATCHECK:
             tmp = StringIndex(arg, GroundingStratNames);
-            if(tmp <= 0)
+            if(tmp < 0)
             {
                DStr_p err = DStrAlloc();
                DStrAppendStr(err,
                              "Wrong argument to option --sat-check. Possible "
                              "values: ");
-               DStrAppendStrArray(err, GroundingStratNames+1, ", ");
+               DStrAppendStrArray(err, GroundingStratNames, ", ");
                Error(DStrView(err), USAGE_ERROR);
                DStrFree(err);
             }
@@ -2000,17 +2002,16 @@ CLState_p process_options(int argc, char* argv[])
       case OPT_FREE_OBJECTS:
             free_symb_prop = free_symb_prop|FPIsObject;
             break;
-      case OPT_DEF_CNF_OLD:
-            new_cnf = false;
-            /* Intentional fall-through */
       case OPT_DEF_CNF:
-            h_parms->formula_def_limit = CLStateGetIntArgCheckRange(handle, arg, 0, LONG_MAX);
+            h_parms->formula_def_limit =
+               CLStateGetIntArgCheckRange(handle, arg, 0, LONG_MAX);
             break;
       case OPT_FOOL_UNROLL:
             h_parms->fool_unroll = CLStateGetBoolArg(handle, arg);
             break;
       case OPT_MINISCOPE_LIMIT:
-            miniscope_limit =  CLStateGetIntArgCheckRange(handle, arg, 0, LONG_MAX);
+            h_parms->miniscope_limit =
+               CLStateGetIntArgCheckRange(handle, arg, 0, LONG_MAX);
             break;
       case OPT_PRINT_TYPES:
             TermPrintTypes = true;
@@ -2256,7 +2257,9 @@ E " VERSION " \"" E_NICKNAME "\"\n\
 \n\
 Usage: " NAME " [options] [files]\n\
 \n\
-Read a set of first-order clauses and formulae and try to refute it.\n\
+Read a set of first-order (or, in the -ho-version, higher-order)\n\
+clauses and formulae and try to prove the conjecture (if given)\n\
+or show the set unsatisfiable.\n\
 \n");
    PrintOptions(stdout, opts, "Options:\n\n");
    fprintf(out, "\n\n" E_FOOTER);
