@@ -63,7 +63,7 @@ typedef enum
    FPPseudoPred   = 8192, /* Pseudo-predicate used for side effects
                            * only, does not conceptually contribute to
                            * truth of clause */
-   FPTypedApplication = FPPseudoPred * 2, /* Symbol used to represtend typed
+   FPTypedApplication = FPPseudoPred * 2, /* Symbol used to represent typed
                                              first-order binary application symbol */
    FPIsInjDefSkolem = FPTypedApplication * 2, /* Symbol is Skolem for injective function */
    FPSkolemSymbol = FPIsInjDefSkolem * 2, /* Obvious ;-) */
@@ -96,7 +96,8 @@ Type_p TypeCheckArithConv(struct sigcell *sig, struct termcell *t);
 typedef struct funccell
 {
    /* f_code is implicit by position in the array */
-   char*  name;
+   char*  name;     // "real" name
+   char*  pname;    // Used for external printing
    int    arity;
    int    alpha_rank; /* We sometimes need an arbitrary but stable
                          order on symbols and use alphabetic. */
@@ -156,6 +157,7 @@ typedef struct sigcell
    FunCode   xor_code;
    /* And here are codes for interpreted symbols */
    FunCode   answer_code;       /* For answer literals */
+   FunCode   distinct_code;     /* $distinct */
 
    /* Sort and type banks (type => sort, but a shortcut is useful) */
    TypeBank_p  type_bank;
@@ -371,7 +373,7 @@ static inline int SigFindArity(Sig_p sig, FunCode f_code)
 //
 /----------------------------------------------------------------------*/
 
-static inline char*  SigFindName(Sig_p sig, FunCode f_code)
+static inline char* SigFindName(Sig_p sig, FunCode f_code)
 {
    if(!f_code)
    {
@@ -380,7 +382,7 @@ static inline char*  SigFindName(Sig_p sig, FunCode f_code)
    assert(f_code > 0);
    assert(f_code <= sig->f_count);
 
-   return (sig->f_info[f_code]).name;
+   return (sig->f_info[f_code]).pname;
 }
 
 
